# IBM Cloud Private CLI

[![Build Status](https://travis.ibm.com/IBMPrivateCloud/cloudctl.svg?token=K4fZzZF5xZgqn9xKsG9D&branch=master)](https://travis.ibm.com/IBMPrivateCloud/cloudctl)
[![Codacy Badge](https://codacy.whitewater.ibm.com:16006/project/badge/Grade/e73caa0e65c64d818f878aff1d4dc7a4)](https://codacy.whitewater.ibm.com:443/app/rws/cloudctl?utm_source=github.ibm.com&amp;utm_medium=referral&amp;utm_content=IBMPrivateCloud/cloudctl&amp;utm_campaign=Badge_Grade)
[![Codacy Badge](https://codacy.whitewater.ibm.com:16006/project/badge/Coverage/e73caa0e65c64d818f878aff1d4dc7a4)](https://codacy.whitewater.ibm.com:443/app/rws/cloudctl?utm_source=github.ibm.com&utm_medium=referral&utm_content=IBMPrivateCloud/cloudctl&utm_campaign=Badge_Coverage)

## Prerequisites

- Must build on Unix or Mac.
- Must have [Go](https://golang.org/dl/) version 1.11.
- Required environment variables:

    ``` shell
    GITHUB_USER=<W3_USER>
    GITHUB_TOKEN=<GITHUB_PERSONAL_ACCESS_TOKEN>
    ARTIFACTORY_USER=<W3_USER>
    ARTIFACTORY_KEY=<ARTIFACTORY_TOKEN>
    ARTIFACTORY_URL=https://na.artifactory.swg-devops.com/artifactory/hyc-cloud-private-scratch-generic-local
    ```

## Targets

**Running the target `deps` is only required once unless dependencies have changed.**

### Binary

Build the cloudctl binary for your local architecture. Use `build` instead of `build-local` to build binaries for all supported architectures.

``` shell
make deps build-local
```

*Binaries will be created in the project's `bin` directory and `build-local` will copy it to `/usr/local/bin`.*

### Usage

The current supported commands are listed below.

``` shell
NAME:
   cloudctl - A command line tool to interact with IBM Cloud Private

USAGE:
[environment variables] cloudctl [global options] command [arguments...] [command options]

VERSION:
   3.1.2-dev+52c3fd9dddfbe2c85487683baca6127dd5ee59b0

COMMANDS:
   api          View the API endpoint and API version for the service.
   catalog      Manage catalog
   cm           Manage cluster
   completion   Generate an autocompletion script for the specified shell (bash or zsh).
   config       Write default values to the configuration.
   helm-init    Prints the configuration of the HELM_HOST setting for helm.
   iam          Manage identities and access to resources
   login        Log user in.
   logout       Log user out.
   metering     Download Metering reports
   plugin       Manage plugins
   pm           Manage passwords
   target       Set or view the targeted namespace.
   tokens       Display the oauth tokens for the current session. Run `cloudctl login` to retrieve the tokens.
   version      Check CLI and API version compatibility.
   help

Enter 'cloudctl help [command]' for more information about a command.

ENVIRONMENT VARIABLES:
   CLOUDCTL_COLOR=false                     Do not colorize output
   CLOUDCTL_HOME=path/to/dir                Path to config directory
   CLOUDCTL_TRACE=true                      Print API request diagnostics to stdout
   CLOUDCTL_TRACE=path/to/trace.log         Append API request diagnostics to a log file

GLOBAL OPTIONS:
   --help, -h                         Show help
```

### Test

To run unit tests with code coverage (in file `coverage/cover.html`):

``` shell
make deps test coverage
```

When adding tests, which is anytime you add code, you need to follow several guidelines to make the code testable.

1. If you are adding a struct that has a func on it, you should be creating an interface and not exposing any data directly.

    ``` go
    package example

    // ThingFactory Factory for Thing instances
    type ThingFactory interface {
        NewThing() Thing
    }

    // production implementaton of ThingFactory
    type thingFactory struct{}

    func (f *thingFactory) NewThing() Thing {
        return &thing{}
    }

    // NewThingFactory Create a new ThingFactory
    func NewThingFactory() ThingFactory {
        return &thingFactory{}
    }
    ```

2. After adding an interface, run `make mocks` to generate mocks of the interface to `internal/mocks`.
    - Code in `internal/mocks` should never be hand-edited, but it should be committed.
    - Code under `internal` in Go is accessible to your project but is not packaged into the binary.
    - This is how we make mocks available to all tests but not ship them.
3. Test files should either be black box tests (`*_test.go`) or internal tests (`*_internal_test.go`).
    - Internal tests need to be in the same package as the code under test.
    - Black box tests should be in the same folder but a different package. A black box test file in directory/package `foo` would have `package foo_test`.
    - Black box tests can dot-import the package under test and rename packages with mocks:

        ``` go
        package example_test

        import (
            "testing"

            mock_example "github.ibm.com/IBMPrivateCloud/testing/internal/mocks/example"
            . "github.ibm.com/IBMPrivateCloud/testing/example"
        )
        ```

## Translation

The project has a [unit test](plugin/i18n/i18n_test.go) to verify that all calls to `i18n.T()` have the translated key in [cli/i18n/resources/en_US.all.json](cli/i18n/resources/en_US.all.json). There is also a script to find and add all untranslated keys to the file:

``` shell
make i18n-fix
```
